@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: macros.h 1009 2006-02-14 16:11:43Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -35,6 +35,8 @@
 #include <pthread.h>
 #endif
 
+#include <stdbool.h>
+
 #define LINEBUFSIZE     	2048
 
 #ifndef MAXHOSTNAMELEN
@@ -55,8 +57,10 @@
 #endif
 
 #ifndef _BOOL_DEFINED
-#define _BOOL_DEFINED
+#  define _BOOL_DEFINED
+#  if !defined (true) && !defined (false)
 typedef enum { false, true } bool;
+#  endif
 #endif
 
 #endif                          /* !_MACROS_INCLUDED */
