@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: mcmd.c 1151 2008-01-29 17:45:54Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -250,7 +250,8 @@ mcmd_signal(int fd, void *arg, int signum)
         if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0)
             err("%p: fcntl: %m\n");
         c = (char) signum;
-        write(fd, &c, 1);
+        if (write(fd, &c, 1) < 0)
+            err("%p: write (signalfd): %m\n");
     }
     return 0;
 }
@@ -529,7 +530,10 @@ mcmd(char *ahost, char *addr, char *locuser, char *remuser, char *cmd,
             goto bad;
         }
     } else {
-        write(s, "", 1);
+        if (write(s, "", 1) != 1) {
+            err ("%p: %S: mcmd: write NUL stderr port: %m\n", ahost);
+            goto bad;
+        }
         lport = 0;
     }
 
@@ -620,7 +624,7 @@ mcmd(char *ahost, char *addr, char *locuser, char *remuser, char *cmd,
 
             memcpy(tptr,(char *) &rand,sizeof(rand));
             tptr += sizeof(rand);
-            if (fd_read_line (s3, tptr, LINEBUFSIZE) < 0)
+            if (fd_read_line (s3, tptr, LINEBUFSIZE - sizeof(rand)) < 0)
                 err("%p: %S: mcmd: Read error from remote host: %m\n", ahost);
             else
                 err("%p: %S: mcmd: Error: %s\n", ahost, &tmpbuf[0]);
