@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: netgroup.c 1084 2006-12-05 00:09:07Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2005-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -84,7 +84,7 @@ struct pdsh_module_option netgroup_module_options[] =
    PDSH_OPT_TABLE_END
  };
 
-/* 
+/*
  * Machines module info 
  */
 struct pdsh_module pdsh_module_info = {
@@ -119,21 +119,17 @@ static hostlist_t _read_netgroup (const char *group)
 {
 	hostlist_t hl = NULL;
 	char *host, *user, *domain;
-	char buf[4096];
     int rc;
 
-	if (setnetgrent (group) < 0) {
-		err ("%p: netgroup \"%s\": %m\n");
-		return NULL;
-	}
+	setnetgrent (group);
 
-	while (rc = getnetgrent_r (&host, &user, &domain, buf, sizeof (buf))) {
+	while ((rc = getnetgrent (&host, &user, &domain))) {
 		if (hl == NULL)
 			hl = hostlist_create (host);
 		else
 			hostlist_push (hl, host);
 	}
-	
+
 	endnetgrent ();
 
 	return (hl);
