@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: sshcmd.c 1178 2009-03-24 21:46:02Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -70,13 +70,13 @@
 #include "src/pdsh/dsh.h"
 #include "src/pdsh/mod.h"
 
-#define HBUF_LEN	1024
-
 #if STATIC_MODULES
 #  define pdsh_module_info sshcmd_module_info
 #  define pdsh_module_priority sshcmd_module_priority
 #endif    
 
+#define DEFAULT_SSH_ARGS "-2 -a -x %h"
+
 int pdsh_module_priority = DEFAULT_MODULE_PRIORITY;
 
     
@@ -88,6 +88,7 @@ static int sshcmd_signal(int, void *arg, int);
 static int sshcmd(char *, char *, char *, char *, char *, int, int *, void **);
 static int sshcmd_destroy (pipecmd_t p);
 static int sshcmd_args_init (void);
+static int fixup_ssh_args (List ssh_args_list, int need_user);
 
 List ssh_args_list =     NULL;
 
@@ -134,25 +135,31 @@ struct pdsh_module pdsh_module_info = {
   &sshcmd_module_options[0],
 };
 
-static char **ssh_argv_create (const char *cmd)
+static char **ssh_argv_create (List arg_list, const char **remote_argv)
 {
     int n;
     char *arg;
     char **argv;
+    const char **p;
     ListIterator i;
 
-    n = list_count (ssh_args_list) + 4;
+    n = 0;
+    for (p = remote_argv; *p; p++)
+        n++;
+
+    n += list_count (arg_list) + 2;
     argv = (char **) Malloc (n * sizeof (char *));
     memset (argv, 0, n);
 
     n = 0;
-    i = list_iterator_create (ssh_args_list);
+    i = list_iterator_create (arg_list);
     while ((arg = list_next (i))) 
         argv[n++] = Strdup (arg);
     list_iterator_destroy (i);
 
-    /* Append command to standard list of args */
-    argv[n] = Strdup (cmd);
+    /* Append remote_argv to standard list of args */
+    for (p = remote_argv; *p; p++)
+        argv[n++] = Strdup (*p);
 
     return (argv);
 }
@@ -183,6 +190,13 @@ static int mod_ssh_postop(opt_t *opt)
 {
     sshcmd_args_init ();
     ssh_args_prepend_timeout (opt->connect_timeout);
+
+    /*
+     *  Append PATH=...; to ssh args if DSHPATH was set
+     */
+    if (opt->dshpath)
+        list_append (ssh_args_list, Strdup (opt->dshpath));
+
     return 0;
 }
 
@@ -191,8 +205,10 @@ static int sshcmd_init(opt_t * opt)
     /*
      * Drop privileges if running setuid root
      */
-    if ((geteuid() == 0) && (getuid() != 0))
-        setuid (getuid ());
+    if ((geteuid() == 0) && (getuid() != 0)) {
+        if (setuid (getuid ()) < 0)
+            errx ("%p: setuid: %m\n");
+    }
 
     /*
      *  Do not resolve hostnames in pdsh when using ssh
@@ -203,6 +219,25 @@ static int sshcmd_init(opt_t * opt)
     return 0;
 }
 
+static void free_f (void *x)
+{
+    Free ((void **) &x);
+}
+
+static List ssh_args_list_copy (List args)
+{
+    List copy;
+    ListIterator i = list_iterator_create (args);
+    const char *arg;
+
+    copy = list_create ((ListDelF) free_f);
+    while ((arg = list_next (i)))
+        list_append (copy, Strdup (arg));
+    list_iterator_destroy (i);
+
+    return (copy);
+}
+
 static int mod_ssh_exit (void)
 {
     if (ssh_args_list)
@@ -231,7 +266,38 @@ sshcmd(char *ahost, char *addr, char *luser, char *ruser, char *cmd,
          int rank, int *fd2p, void **arg)
 {
     pipecmd_t p = NULL;
-    char **ssh_args = ssh_argv_create (cmd);
+    const char **remote_argv = pdsh_remote_argv ();
+    const char *alt_argv[] = { cmd, NULL };
+    char **ssh_args;
+    List args_list;
+
+    /*
+     *  If running as pdcp/rpdcp, then the dsh code has rewritten
+     *   the cmd to invoke pdcp server on remote nodes. Therefore
+     *    avoid using pdsh_remote_argv() directly, instead use cmd:
+     */
+    if (pdsh_personality() == PCP)
+        remote_argv = alt_argv;
+    /*
+     *  In interactive dsh mode, pdsh_remote_argv() will be empty
+     *   so we can't use it.
+     */
+    if (!remote_argv || !*remote_argv)
+        remote_argv = alt_argv;
+
+    /*
+     *  Create a copy of ssh_args_list to customize for this target
+     */
+    args_list = ssh_args_list_copy (ssh_args_list);
+
+    if (strcmp (luser, ruser) != 0)
+        fixup_ssh_args (args_list, 1);
+    else
+        fixup_ssh_args (args_list, 0);
+
+    ssh_args = ssh_argv_create (args_list, remote_argv);
+
+    list_destroy (args_list);
 
     if (!(p = pipecmd ("ssh", (const char **) ssh_args, ahost, ruser, rank)))
         goto out;
@@ -259,27 +325,82 @@ sshcmd_destroy (pipecmd_t p)
     return WEXITSTATUS (status);
 }
 
+/*
+ *  Check string argument [arg] for parameter substitution sequence [s].
+ *    If [s] is found in [arg] then also check that [s] is not preceeded
+ *    by '%' which would have the effect of escaping the parameter
+ *    substitution.
+ */
+static int arg_has_parameter (const char *arg, const char *s)
+{
+    char *p;
+    if ((p = strstr (arg, s)) && ((p == arg) || (*(p-1) != '%')))
+        return 1;
+    return 0;
+}
+
+/*
+ *  Scan the current ssh_args_list for presence of %u and %h.
+ *   If they are not present, assume we need to append them to the
+ *   ssh args.
+ */
+static int fixup_ssh_args (List ssh_args_list, int need_user)
+{
+    ListIterator i = list_iterator_create (ssh_args_list);
+    char *arg;
+    int got_user = 0;
+    int got_host = 0;
+
+    while ((arg = list_next (i))) {
+        if (need_user && arg_has_parameter (arg, "%u"))
+            got_user = 1;
+        if (arg_has_parameter (arg, "%h"))
+            got_host = 1;
+    }
+
+    if (need_user && !got_user) {
+        if (got_host) {
+            /*
+             *  Ensure "%lu" is not inserted after "%h":
+             */
+            list_iterator_reset (i);
+            arg = list_find (i, (ListFindF) arg_has_parameter, "%h");
+            list_insert (i, Strdup ("-l%u"));
+        }
+        else
+            list_append (ssh_args_list, Strdup ("-l%u"));
+    }
+
+    /*
+     *  Always append "%h" to end of args
+     */
+     if (!got_host)
+         list_append (ssh_args_list, Strdup ("%h"));
+
+     list_iterator_destroy (i);
+     return (0);
+}
 
 static int sshcmd_args_init (void)
 {
     char *val = NULL;
     char *str = NULL;
-    List args_list = NULL;
-
-    if (!(val = getenv ("PDSH_SSH_ARGS"))) 
-        val = "-2 -a -x -l%u %h";
-
-    str = Strdup (val);
-    args_list = list_split (" ", str);
-    Free ((void **) &str);
 
+    /*
+     *  Place SSH_ARGS_APPEND at the beggining of the args string
+     *   because %h must always come last...
+     */
     if ((val = getenv ("PDSH_SSH_ARGS_APPEND"))) {
         str = Strdup (val);
-        args_list = list_split_append (args_list, " ", str);
-        Free ((void **) &str);
+        xstrcatchar (&str, ' ');
     }
 
-    ssh_args_list = args_list;
+    if (!(val = getenv ("PDSH_SSH_ARGS")))
+        val = DEFAULT_SSH_ARGS;
+    xstrcat (&str, val);
+
+    ssh_args_list = list_split (" ", str);
+    Free ((void **) &str);
 
     return (0);
 }
