@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: mod.c 1151 2008-01-29 17:45:54Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -41,7 +41,7 @@
 #if STATIC_MODULES
 #include "static_modules.h"
 #else
-#include "ltdl.h"
+#include "dlfcn.h"
 #endif 
 
 #include "src/common/err.h"
@@ -49,6 +49,7 @@
 #include "src/common/xstring.h"
 #include "src/common/hostlist.h"
 #include "src/common/list.h"
+#include "src/common/split.h"
 #include "mod.h"
 
 /*
@@ -71,12 +72,13 @@ struct module_components {
 #endif
 
 #if !STATIC_MODULES
-    lt_dlhandle handle;
+    void *handle;
 #endif
     char *filename;
 
     int priority;
- 
+    int initialized;
+
     struct pdsh_module *pmod;
 };
 
@@ -103,6 +105,8 @@ static int  _is_loaded(char *filename);
 static bool _path_permissions_ok(const char *dir, uid_t pdsh_owner);
 static perm_error_t  _dir_permission_error(struct stat *, uid_t alt_uid);
 #endif
+static int  _mod_initialize(mod_t mod);
+static int  _mod_init_list_safe(mod_t mod, void *arg);
 static void _mod_destroy(mod_t mod);
 static bool _mod_opts_ok(mod_t mod);
 static int  _mod_print_info(mod_t mod);
@@ -110,27 +114,21 @@ static void _print_option_help(struct pdsh_module_option *p, int col);
 static struct pdsh_module_option * _mod_find_opt(mod_t mod, int opt);
 
 /*
- *  Static list of loaded modules and persistent iterator
+ *  Static list of loaded modules
  */
 static List module_list;
-static ListIterator module_itr;
 static bool initialized = false;
 
 int
 mod_init(void)
 {
     if (!initialized) {
-        if ( !(module_list = list_create((ListDelF) _mod_destroy))
-           || !(module_itr = list_iterator_create(module_list))  ) {
-            err("Unable to create module list and iterator\n");
+        if (!(module_list = list_create((ListDelF) _mod_destroy))) {
+            err("Unable to create module list\n");
             return -1;
         }
         initialized = true;
-#if STATIC_MODULES
         return 0;
-#else
-        return lt_dlinit();
-#endif
     } else
         return 0;
 }
@@ -145,14 +143,9 @@ mod_exit(void)
      *  list_destroy() will call module destructor on each 
      *    element in list
      */
-    list_iterator_destroy(module_itr);
     list_destroy(module_list);
     
-#if STATIC_MODULES || PREVENT_DLCLOSE_BUG
     return 0;
-#else
-    return lt_dlexit();
-#endif
 }
 
 hostlist_t
@@ -174,6 +167,22 @@ _mod_postop(mod_t mod, opt_t *pdsh_opts)
 }
 
 
+/*
+ *  Like list_next (i), but skip over inactive modules
+ */
+static mod_t _mod_next_active (ListIterator i)
+{
+    mod_t mod;
+
+    while ((mod = list_next (i))) {
+        if (mod->initialized)
+            return (mod);
+    }
+
+    return (NULL);
+}
+
+
 /*
  *  Call any "read wcoll" functions exported by modules. The module
  *    is responsible for deciding when to generate a new wcoll, 
@@ -189,12 +198,17 @@ int
 mod_read_wcoll(opt_t *opt)
 {
     mod_t mod;
+    ListIterator module_itr;
 
     if (!initialized)
         mod_init();
 
-    list_iterator_reset(module_itr);
-    while ((mod = list_next(module_itr))) {
+    if (!(module_itr = list_iterator_create(module_list))) {
+        err("Unable to create module list iterator\n");
+        return -1;
+    }
+
+    while ((mod = _mod_next_active (module_itr))) {
         hostlist_t hl = NULL;
 
         if (!(hl = _mod_read_wcoll(mod, opt)))
@@ -207,6 +221,8 @@ mod_read_wcoll(opt_t *opt)
             opt->wcoll = hl;
     }
 
+    list_iterator_destroy(module_itr);
+
     return 0;
 }
 
@@ -215,14 +231,21 @@ mod_postop(opt_t *pdsh_opts)
 {
     mod_t mod;
     int errors = 0;
+    ListIterator module_itr;
 
     if (!initialized)
         mod_init();
 
-    list_iterator_reset(module_itr);
-    while ((mod = list_next(module_itr)))
+    if (!(module_itr = list_iterator_create(module_list))) {
+        err("Unable to create module list iterator\n");
+        return 1;
+    }
+
+    while ((mod = _mod_next_active (module_itr)))
         errors += _mod_postop(mod, pdsh_opts);
 
+    list_iterator_destroy(module_itr);
+
     return errors;
 }
 
@@ -237,8 +260,8 @@ mod_create(void)
     mod->filename = NULL;
 
     mod->priority = DEFAULT_MODULE_PRIORITY;
+    mod->initialized = 0;
     assert(mod->magic = MOD_MAGIC);
-    
     return mod;
 }
 
@@ -252,7 +275,11 @@ _mod_destroy(mod_t mod)
         mod->pmod->type = NULL;
         mod->pmod->name = NULL;
 
-        if (mod->pmod->mod_ops && mod->pmod->mod_ops->exit)
+        /*
+         *  Only run exit function if module was initialized
+         */
+        if (mod->initialized &&
+            mod->pmod->mod_ops && mod->pmod->mod_ops->exit)
             (*mod->pmod->mod_ops->exit)();
     }
 
@@ -262,7 +289,7 @@ _mod_destroy(mod_t mod)
 #if !STATIC_MODULES
 #  if !PREVENT_DLCLOSE_BUG
     if (mod->handle)
-        lt_dlclose(mod->handle);
+        dlclose(mod->handle);
 #  endif
 #endif
 
@@ -284,11 +311,46 @@ _mod_opts_ok(mod_t mod)
 static int
 _cmp_f (mod_t x, mod_t y)
 {
+    if (x->priority == y->priority)
+        return strcmp (x->pmod->name, y->pmod->name);
     return (y->priority - x->priority);
 }
 
-int 
-mod_load_modules(const char *dir, opt_t *opt)
+
+static int
+_mod_find_misc (mod_t mod, const char *name)
+{
+    if (strcmp (mod->pmod->type, "misc") != 0)
+        return 0;
+    if (strcmp (mod->pmod->name, name) != 0)
+        return 0;
+    return 1;
+}
+
+static int
+_mod_initialize_by_name (char *name, List l)
+{
+    mod_t mod = list_find_first (l, (ListFindF) _mod_find_misc, name);
+    if (mod != NULL && _mod_initialize (mod) < 0)
+        err("%p: Warning: Failed to initialize requested module \"%s/%s\"\n",
+                mod->pmod->type, mod->pmod->name);
+    return (0);
+}
+
+
+static int _mod_initialize_modules_by_name (char *names, List m)
+{
+    List l;
+
+    if (names == NULL || strlen(names) == 0)
+        return (0);
+
+    l = list_split (",", names);
+    list_for_each (l, (ListForF) _mod_initialize_by_name, m);
+    return (0);
+}
+
+int mod_load_modules(const char *dir, opt_t *opt)
 {
     int rc = 0;
 #if STATIC_MODULES
@@ -299,6 +361,16 @@ mod_load_modules(const char *dir, opt_t *opt)
 
     list_sort(module_list, (ListCmpF) _cmp_f);
 
+    /*
+     *  Initialize misc modules by name
+     */
+    _mod_initialize_modules_by_name (opt->misc_modules, module_list);
+
+    /*
+     *  Initialize remaining modules in modules_list:
+     */
+    list_for_each (module_list, (ListForF) _mod_init_list_safe, NULL);
+
     return(rc);
 }
 
@@ -351,6 +423,7 @@ _mod_print_info(mod_t mod)
     out("Module: %s/%s\n",    mod->pmod->type, mod->pmod->name); 
     out("Author: %s\n",       mod->pmod->author ? mod->pmod->author : "???");
     out("Descr:  %s\n",       mod->pmod->descr ? mod->pmod->descr : "???");
+    out("Active: %s\n",       mod->initialized ? "yes" : "no");
 
     if (mod->pmod->opt_table && mod->pmod->opt_table->opt) {
         out("Options:\n");
@@ -364,7 +437,8 @@ _mod_print_info(mod_t mod)
 
 static int _opt_print(mod_t mod, int *col)
 {
-    mod_print_options(mod, *col);
+    if (mod->initialized)
+        mod_print_options(mod, *col);
     return 0;
 }
 
@@ -385,43 +459,69 @@ int
 mod_count(char *type)
 {
     int i = 0;
+    ListIterator module_itr;
 
     assert(module_list != NULL);
 
     if (type == NULL)
         return list_count(module_list);
 
-    list_iterator_reset(module_itr);
+    if (!(module_itr = list_iterator_create(module_list))) {
+        err("Unable to create module list iterator\n");
+        return -1;
+    }
+
     while (list_find(module_itr, (ListFindF) _cmp_type, type)) 
         i++;
 
+    list_iterator_destroy(module_itr);
+
     return i;
 }
 
-List
-mod_get_module_names(char *type)
+static List
+_mod_get_module_names(char *type, int get_active)
 {
     List l;
     mod_t mod;
+    ListIterator module_itr;
 
     assert(module_list != NULL);
 
     l = list_create(NULL);
-    list_iterator_reset(module_itr);
 
-    if (type == NULL) {
-        while((mod = list_next(module_itr)))
+    if (!(module_itr = list_iterator_create(module_list))) {
+        err("Unable to create module list iterator\n");
+        list_destroy(l);
+        return NULL;
+    }
+
+    while ((mod = (type ? list_find(module_itr, (ListFindF) _cmp_type, type)
+                        : list_next(module_itr)))) {
+        /*
+         *  Push active (initialized) modules onto list if get_active
+         *   is true, otherwise push inactive (!initialized) modules:
+         */
+        if (!get_active == !mod->initialized)
             list_push(l, mod->pmod->name);
-        return l;
     }
 
-    while ((mod = list_find(module_itr, (ListFindF) _cmp_type, type))) {
-        list_push(l, mod->pmod->name);
-    }
+    list_iterator_destroy(module_itr);
 
     return l;
 }
 
+List mod_get_module_names (char *type)
+{
+    return _mod_get_module_names (type, 1);
+}
+
+List mod_get_uninitialized_module_names (char *type)
+{
+    return _mod_get_module_names (type, 0);
+}
+
+
 void
 mod_list_module_info(void)
 {
@@ -449,16 +549,24 @@ mod_t
 mod_get_module(const char *type, const char *name)
 {
     mod_t mod;
+    ListIterator module_itr;
 
     assert(type != NULL);
     assert(name != NULL);
 
-    list_iterator_reset(module_itr);
+    if (!(module_itr = list_iterator_create(module_list))) {
+        err("Unable to create module list iterator\n");
+        return NULL;
+    }
+
     while ((mod = list_next(module_itr))) {
         if (_mod_description_match (mod, type, name))
-            return mod;
+            break;
     }
-    return NULL;
+
+    list_iterator_destroy(module_itr);
+
+    return mod;
 }
 
 char *
@@ -533,12 +641,22 @@ mod_process_opt(opt_t *opt, int c, char *optarg)
 {
     mod_t mod;
     struct pdsh_module_option *p = NULL;
+    ListIterator module_itr;
 
-    list_iterator_reset(module_itr);
-    while ((mod = list_next(module_itr))) {
-        if ((p = _mod_find_opt(mod, c)))
-            return p->f(opt, c, optarg);
+    if (!(module_itr = list_iterator_create(module_list))) {
+        err("Unable to create module list iterator\n");
+        return -1;
     }
+
+    while ((mod = _mod_next_active (module_itr))) {
+        if ((p = _mod_find_opt(mod, c))) {
+            list_iterator_destroy(module_itr);
+            return p->f(opt, c, optarg);
+        }
+    }
+
+    list_iterator_destroy(module_itr);
+
     return -1;
 }
 
@@ -570,8 +688,7 @@ _mod_delete (const char *type, const char *name)
 }
 
 
-static int 
-_mod_install(mod_t mod, const char *name)
+static int _mod_register (mod_t mod, const char *name)
 {
     mod_t prev;
     /* 
@@ -602,11 +719,15 @@ _mod_install(mod_t mod, const char *name)
     if (!(mod->pmod->personality & pdsh_personality()))
         return -1;
 
-    if (!_mod_opts_ok(mod)) {
-        err("failed to install module options for \"%s/%s\"\n", 
-            mod->pmod->type, mod->pmod->name);
+    list_prepend(module_list, mod);
+
+    return 0;
+}
+
+static int _mod_initialize (mod_t mod)
+{
+    if (!_mod_opts_ok(mod))
         return -1;
-    }
 
     if (mod->pmod->mod_ops && 
         mod->pmod->mod_ops->init && 
@@ -616,11 +737,21 @@ _mod_install(mod_t mod, const char *name)
         return -1;
     }
 
-    list_prepend(module_list, mod);
+    mod->initialized = 1;
 
     return 0;
 }
 
+/*
+ *  Version of _mod_initialize that always returns zero
+ *   for use with list_for_each.
+ */
+static int _mod_init_list_safe (mod_t mod, void *arg)
+{
+    _mod_initialize (mod);
+    return 0;
+}
+
 
 #if STATIC_MODULES
 /*
@@ -636,10 +767,7 @@ _mod_load_static(int idx)
     mod->priority = *priority[idx];
     mod->filename = Strdup("static");
 
-    if (_mod_install(mod, static_mod_names[idx]) < 0) {
-        _mod_destroy(mod);
-        return -1;
-    }
+    _mod_register(mod, static_mod_names[idx]);
 
     return 0;
 }
@@ -670,22 +798,21 @@ static int
 _mod_load_dynamic(const char *fq_path)
 {
     mod_t mod = NULL;
-    const lt_dlinfo *info;
     int *priority;
     assert(fq_path != NULL);
 
     mod = mod_create();
 
-    if (!(mod->handle = lt_dlopen(fq_path)))
+    /*  RTLD_GLOBAL is used here so that symbols in any libraries
+     *  linked to by the loaded module are global, such that any DSOs
+     *  dlopened in turn by those libraries can find symbols in
+     *  their parent library. This is specifically a problem with the
+     *  nodeupdown and slurm modules at this time.
+     */
+    if (!(mod->handle = dlopen(fq_path, RTLD_GLOBAL | RTLD_NOW)))
         goto fail;
 
-    if (!(info = lt_dlgetinfo(mod->handle))) 
-        goto fail_libtool_broken;
-
-    if (info->filename == NULL) 
-        goto fail_libtool_broken;
-
-    mod->filename = Strdup(info->filename);
+    mod->filename = Strdup(fq_path);
 
     if (_is_loaded(mod->filename)) {
         /* Module already loaded. This is OK, no need for
@@ -696,25 +823,19 @@ _mod_load_dynamic(const char *fq_path)
     }
   
     /* load all module info from the pdsh_module structure */
-    if (!(mod->pmod = lt_dlsym(mod->handle, "pdsh_module_info"))) {
+    if (!(mod->pmod = dlsym(mod->handle, "pdsh_module_info"))) {
         err("%p:[%s] can't resolve pdsh module\n", mod->filename);
         goto fail;
     }
 
-    if ((priority = lt_dlsym(mod->handle, "pdsh_module_priority"))) 
+    if ((priority = dlsym(mod->handle, "pdsh_module_priority"))) 
         mod->priority = *priority;
-        
-    if (_mod_install(mod, mod->filename) < 0)
+
+    if (_mod_register(mod, mod->filename) < 0)
         goto fail;
 
     return 0;
 
- fail_libtool_broken:
-    /*
-     * Avoid dlclose() of invalid handle
-     */
-    mod->handle = NULL;
-
  fail:
     _mod_destroy(mod);
     return -1;
@@ -751,7 +872,7 @@ _mod_load_dynamic_modules(const char *dir, opt_t *pdsh_opts)
     if (!initialized) 
         mod_init();
 
-    if (_pdsh_owner(pdsh_opts->path_progname, &pdsh_owner) < 0)
+    if (_pdsh_owner(pdsh_opts->local_program_path, &pdsh_owner) < 0)
         return -1;
 
     if (!_path_permissions_ok(dir, pdsh_owner)) 
