@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: mod.h 1009 2006-02-14 16:11:43Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -122,6 +122,12 @@ int mod_count(char *type);
  */
 List mod_get_module_names(char *type);
 
+/*
+ *  Build list of module names that are loaded but not initialized
+ */
+List mod_get_uninitialized_module_names (char *type);
+
+
 /*
  * Print all options provided by modules
  *   Justify option description starting on given column.
