@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: pcp_server.c 1141 2007-08-22 20:27:04Z achu $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -62,7 +62,7 @@ char copyright[] =
 /*
  * From: @(#)rcp.c	5.32 (Berkeley) 2/25/91
  */
-char rcsid[] = "$Id: pcp_server.c 1141 2007-08-22 20:27:04Z achu $";
+char rcsid[] = "$Id$";
 /* #include "../version.h" */
 
 #if HAVE_CONFIG_H
@@ -180,6 +180,7 @@ _allocbuf(struct pcp_server *s, BUF *bp, int fd, int blksize)
         bp->buf = malloc(size);
         if (!bp->buf) {
             _error(s, "malloc: out of memory\n");
+            bp->cnt = 0;
             return NULL;
         }
     }
@@ -218,15 +219,20 @@ _sink(struct pcp_server *svr, char *targ, BUF *bufp) {
     BUF *bp;
     off_t i, j, size;
     char ch;
-    const char *why;
-    int amt, count, exists, first, mask, mode;
+    const char *why = "failed to set 'why' string";
+    int amt, count, exists, mask, mode;
     int ofd, setimes, targisdir, cursize = 0;
-    char *np, buf[BUFSIZ], *namebuf = NULL;
+    char *np, *buf = NULL, *namebuf = NULL;
 
 #define	atime	tv[0]
 #define	mtime	tv[1]
 #define	SCREWUP(str)	{ why = str; goto screwup; }
 
+    if (!(buf = malloc(BUFSIZ))) {
+        _error(svr, "out of memory for buf: %m\n");
+        return;
+    }
+
     setimes = targisdir = 0;
     mask = umask(0);
     if (!svr->preserve)
@@ -237,18 +243,16 @@ _sink(struct pcp_server *svr, char *targ, BUF *bufp) {
             return;
     }
 
-    (void)write(svr->outfd, "", 1);
+    if (write(svr->outfd, "", 1) != 1)
+        SCREWUP("write failed");
     if (stat(targ, &stb) == 0 && (stb.st_mode & S_IFMT) == S_IFDIR)
         targisdir = 1;
 
-    for (first = 1;; first = 0) {
+    while (1) {
 		int rc;
         cp = buf;
-        if ((rc = read(svr->infd, cp, 1)) <= 0) {
-            if (namebuf)
-                free(namebuf);
-            return;
-        }
+        if ((rc = read(svr->infd, cp, 1)) <= 0)
+            goto end_server;
         if (*cp++ == '\n')
             SCREWUP("unexpected <newline>");
 
@@ -266,10 +270,9 @@ _sink(struct pcp_server *svr, char *targ, BUF *bufp) {
         }
 
         if (buf[0] == 'E') {
-            (void)write(svr->outfd, "", 1);
-            if (namebuf)
-                free(namebuf);
-            return;
+            if (write(svr->outfd, "", 1) != 1)
+                SCREWUP("write failed");
+            goto end_server;
         }
 
         if (ch == '\n')
@@ -292,7 +295,8 @@ _sink(struct pcp_server *svr, char *targ, BUF *bufp) {
             getnum(atime.tv_usec);
             if (*cp++ != '\0')
                 SCREWUP("atime.usec not delimited");
-            (void)write(svr->outfd, "", 1);
+            if (write(svr->outfd, "", 1) != 1)
+                SCREWUP("write failed");
             continue;
         }
         if (*cp != 'C' && *cp != 'D')
@@ -333,6 +337,7 @@ _sink(struct pcp_server *svr, char *targ, BUF *bufp) {
               
                 if (!(namebuf = malloc(need))) { 
                     _error(svr, "out of memory\n");
+                    cursize = 0;
 
                     /* original rcp may not work with a continue here,
                      * but it will work with pdcp protocol.
@@ -380,7 +385,8 @@ bad:
         if (exists && svr->preserve)
             (void)fchmod(ofd, mode);
 
-        (void)write(svr->outfd, "", 1);
+        if (write(svr->outfd, "", 1) != 1)
+            _error(svr, "failed to write to outfd: %m\n");
         if ((bp = _allocbuf(svr, bufp, ofd, BUFSIZ)) == NULL) {
             (void)close(ofd);
             continue;
@@ -430,19 +436,22 @@ bad:
                 _error(svr, "%s: %m\n", np);
                 break;
             case NO:
-                (void)write(svr->outfd, "", 1);
+                if (write(svr->outfd, "", 1) != 1)
+                    _error(svr, "write failed to outfd: %m\n");
                 break;
             case DISPLAYED:
                 break;
         }
     }
 
-    return;
-
 screwup:
     _error(svr, "protocol screwup: %s\n", why);
 
 end_server:
+    if (buf)
+        free(buf);
+    if (namebuf)
+        free(namebuf);
     return;
 }
 
