@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: privsep.c 1080 2006-12-04 19:05:36Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -58,6 +58,30 @@ gid_t user_gid = -1;
 uid_t priv_uid = -1;
 gid_t priv_gid = -1;
 
+/*
+ *  Top 16 bits of port sent to privsep server is used to encode
+ *   the address family (for rresvport_af()). Currently, value is
+ *   either AF_INET AF_INET6 (if zero, then AF_INET).
+ */
+static int privsep_get_family (int *lport)
+{
+	int family = (*lport>>16) & 0xffff;
+	/*
+	 *  Mask out family from lport:
+	 */
+	*lport &= 0xffff;
+
+	return (family ? family : AF_INET);
+}
+
+static int privsep_set_family (int *lport, int family)
+{
+	if (family > 0xffff)
+		return (-1);
+	*lport |= (family<<16);
+	return (0);
+}
+
 static int create_socketpair (void)
 {
 	int pfds[2];
@@ -81,11 +105,12 @@ static void drop_privileges (void)
 	priv_gid = getegid ();
 
 #ifdef _POSIX_SAVED_IDS
-	seteuid (user_uid);
-	setegid (user_gid);
+	if (seteuid (user_uid) < 0 || setegid (user_gid) < 0)
+		errx ("%p: seteuid/setegid: %m\n");
 #else
-	setreuid (priv_uid, user_uid);
-	setregid (priv_gid, user_gid);
+	if (setreuid (priv_uid, user_uid) < 0
+	  || setregid (priv_gid, user_gid < 0))
+		errx ("%p: setreuid/setregid: %m\n");
 #endif
 }
 
@@ -180,6 +205,21 @@ static int recv_rresvport (int pipefd, int *lptr)
 	return (fd);
 }
 
+
+static int p_rresvport_af (int *port, int family)
+{
+#if HAVE_RRESVPORT_AF
+	return rresvport_af (port, family);
+#else
+	/*  Family must be AF_INET
+	 */
+	if (family != AF_INET)
+		err ("%p: rresvport called with family != AF_INET!\n");
+	/* ignore family != AF_INET */
+	return rresvport (port);
+#endif
+}
+
 static int privsep_server (void)
 {
 	int rc;
@@ -190,8 +230,9 @@ static int privsep_server (void)
 	 * for each request on server_fd create a reserved port and
 	 *   send the created fd back to the client.
 	 */
-	while ((rc = read (server_fd, &lport, sizeof (lport)))) {
-		int s = rresvport (&lport);
+	while ((rc = read (server_fd, &lport, sizeof (lport))) > 0) {
+		int family = privsep_get_family (&lport);
+		int s = p_rresvport_af (&lport, family);
 
 		send_rresvport (server_fd, s, lport);
 
@@ -262,25 +303,37 @@ int privsep_fini (void)
 	return (0);
 }
 
-int privsep_rresvport (int *lport)
+int privsep_rresvport_af (int *lport, int family)
 {
-	int s;
+	int s = -1;
 
 	if (client_fd < 0)
-		return (rresvport (lport));
+		return (p_rresvport_af (lport, family));
+
+	if (privsep_set_family (lport, family) < 0) {
+		err ("%p: privsep_rresvport_af: Invalid family %d\n", family);
+		errno = EINVAL;
+		return (-1);
+	}
 
 	if ((errno = pthread_mutex_lock (&privsep_mutex)))
 		errx ("%p: %s:%d: mutex_lock: %m\n", __FILE__, __LINE__);
 
 	if (write (client_fd, lport, sizeof (*lport)) < 0) {
 		err ("%p: privsep: client write: %m\n");
-		return (-1);
+		goto out;
 	}
 
-    	s = recv_rresvport (client_fd, lport);
+	s = recv_rresvport (client_fd, lport);
 
+out:
 	if ((errno = pthread_mutex_unlock (&privsep_mutex)))
 		errx ("%p: %s:%d: mutex_unlock: %m\n", __FILE__, __LINE__);
 
 	return (s);
 }
+
+int privsep_rresvport (int *lport)
+{
+	return privsep_rresvport_af (lport, AF_INET);
+}
