@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: wcoll.h 1009 2006-02-14 16:11:43Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -33,15 +33,19 @@
 
 #include <stdio.h>              /* FILE *     */
 
+#include "src/common/macros.h"
 #include "src/common/hostlist.h"
 
-#ifndef _BOOL_DEFINED
-#define _BOOL_DEFINED
-typedef enum { false, true } bool;
-#endif /* !_BOOL_DEFINED */
-
 hostlist_t read_wcoll(char *, FILE *);
 
+/*
+ *  Read WCOLL file [file] searching colon-separated path [path].
+ *
+ *  Supports the use of '#include filename' to include other WCOLL files
+ *   that are also in [path].
+ */
+hostlist_t read_wcoll_path (const char *path, const char *file);
+
 #endif
 /*
  * vi:tabstop=4 shiftwidth=4 expandtab
